import { AnyRouter, ClientDataTransformerOptions, inferHandlerInput, inferProcedureInput, inferSubscriptionOutput } from '@trpc/server';
import { Observer, Unsubscribable } from '@trpc/server/observable';
import { TRPCResultMessage } from '@trpc/server/rpc';
import { CancelFn } from '..';
import { TRPCClientError } from '../TRPCClientError';
import { HTTPHeaders, OperationContext, TRPCClientRuntime, TRPCLink } from '../links/types';
declare type CancellablePromise<T = unknown> = Promise<T> & {
    cancel: CancelFn;
};
interface CreateTRPCClientBaseOptions {
    /**
     * Add ponyfill for fetch
     */
    fetch?: typeof fetch;
    /**
     * add ponyfill for AbortController
     */
    AbortController?: typeof AbortController;
    /**
     * headers to be set on outgoing requests / callback that of said headers
     */
    headers?: HTTPHeaders | (() => HTTPHeaders | Promise<HTTPHeaders>);
    /**
     * Data transformer
     * @link https://trpc.io/docs/data-transformers
     **/
    transformer?: ClientDataTransformerOptions;
}
/** @internal */
export interface CreateTRPCClientWithURLOptions extends CreateTRPCClientBaseOptions {
    /**
     * HTTP URL of API
     **/
    url: string;
}
/** @internal */
export interface CreateTRPCClientWithLinksOptions<TRouter extends AnyRouter> extends CreateTRPCClientBaseOptions {
    /**
     * @link https://trpc.io/docs/links
     **/
    links: TRPCLink<TRouter>[];
}
export interface TRPCRequestOptions {
    /**
     * Pass additional context to links
     */
    context?: OperationContext;
}
/** @internal */
export declare type CreateTRPCClientOptions<TRouter extends AnyRouter> = CreateTRPCClientWithLinksOptions<TRouter> | CreateTRPCClientWithURLOptions;
export declare class TRPCClient<TRouter extends AnyRouter> {
    private readonly links;
    readonly runtime: TRPCClientRuntime;
    constructor(opts: CreateTRPCClientOptions<TRouter>);
    private $request;
    private requestAsPromise;
    query<TQueries extends TRouter['_def']['queries'], TPath extends string & keyof TQueries>(path: TPath, ...args: [...inferHandlerInput<TQueries[TPath]>, TRPCRequestOptions?]): CancellablePromise<import("@trpc/server").ThenArg<ReturnType<TQueries[TPath]["call"]>>>;
    mutation<TMutations extends TRouter['_def']['mutations'], TPath extends string & keyof TMutations>(path: TPath, ...args: [...inferHandlerInput<TMutations[TPath]>, TRPCRequestOptions?]): CancellablePromise<import("@trpc/server").ThenArg<ReturnType<TMutations[TPath]["call"]>>>;
    subscription<TSubscriptions extends TRouter['_def']['subscriptions'], TPath extends string & keyof TSubscriptions, TOutput extends inferSubscriptionOutput<TRouter, TPath>, TInput extends inferProcedureInput<TSubscriptions[TPath]>>(path: TPath, input: TInput, opts: TRPCRequestOptions & Partial<Observer<TRPCResultMessage<TOutput>, TRPCClientError<TRouter>>>): Unsubscribable;
}
export {};
//# sourceMappingURL=TRPCClient.d.ts.map