import { AnyRouter, DataTransformer, inferRouterError } from '@trpc/server';
import { Observable, Observer } from '@trpc/server/observable';
import { TRPCResponse, TRPCResponseMessage } from '@trpc/server/rpc';
import { TRPCClientError } from '../TRPCClientError';
export declare type CancelFn = () => void;
export declare type PromiseAndCancel<TValue> = {
    promise: Promise<TValue>;
    cancel: CancelFn;
};
export declare type OperationContext = Record<string, unknown>;
export declare type Operation<TInput = unknown> = {
    id: number;
    type: 'query' | 'mutation' | 'subscription';
    input: TInput;
    path: string;
    context?: OperationContext;
};
export declare type HTTPHeaders = Record<string, string | string[]>;
/**
 * The default `fetch` implementation has an overloaded signature. By convention this library
 * only uses the overload taking a string and options object.
 */
export declare type TRPCFetch = (url: string, options?: RequestInit) => Promise<Response>;
export interface TRPCClientRuntime {
    fetch: TRPCFetch;
    AbortController?: typeof AbortController;
    headers: () => HTTPHeaders | Promise<HTTPHeaders>;
    transformer: DataTransformer;
}
declare type OperationResultData<TRouter extends AnyRouter, TOutput> = TRPCResponse<TOutput, inferRouterError<TRouter>> | TRPCResponseMessage<TOutput, inferRouterError<TRouter>>;
export interface OperationResult<TRouter extends AnyRouter, TOutput> {
    data: OperationResultData<TRouter, TOutput>;
    context?: OperationContext;
}
export declare type OperationResultObservable<TRouter extends AnyRouter, TOutput> = Observable<OperationResult<TRouter, TOutput>, TRPCClientError<TRouter>>;
export declare type OperationResultObserver<TRouter extends AnyRouter, TOutput> = Observer<OperationResult<TRouter, TOutput>, TRPCClientError<TRouter>>;
export declare type OperationLink<TRouter extends AnyRouter, TInput = unknown, TOutput = unknown> = (opts: {
    op: Operation<TInput>;
    next: (op: Operation<TInput>) => OperationResultObservable<TRouter, TOutput>;
}) => OperationResultObservable<TRouter, TOutput>;
export declare type TRPCLink<TRouter extends AnyRouter> = (opts: TRPCClientRuntime) => OperationLink<TRouter>;
export {};
//# sourceMappingURL=types.d.ts.map